-- Dropdown v1.3

DropdownList = class();

local Constants =
{
	Border = 2,
	ScrollBarSize = 10,
	SortIcon =
	{
		Width = 15,
		Height = 15,
		LabelWidth = 20,
		DropdownArrow = "Simbo/UserChatter/Icons/DropdownArrow.tga"
	},
	Colours =
	{
		NormalOutline = Turbine.UI.Color(1.0, 0.8, 0.65, 0.1),
		HoverOutline = Turbine.UI.Color(0.8, 0.65, 0.1),
		SelectedItem = Turbine.UI.Color(0.90, 0.25, 0.25, 0.25)
	}
}

function DropdownList:Constructor(parent)
	self.HeightInRows = nil;
	self.DropdownHeight = nil;
	
	self.DisplayItem = Turbine.UI.Control();
	self.DisplayItem:SetParent(parent);
	
	self.DisplayItem.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self.SelectionList:SetVisible(not self.SelectionList:IsVisible());
			self.SelectionListBox:Focus();
		end
	end
	
	self.DisplayItemBox = Turbine.UI.Label();
	self.DisplayItemBox:SetParent(self.DisplayItem);
	self.DisplayItemBox:SetMouseVisible(false);
	self.DisplayItemBox:SetPosition(Constants.Border, Constants.Border);
	self.DisplayItemBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	
	self.SelectIconLabel = Turbine.UI.Label();
	self.SelectIconLabel:SetParent(self.DisplayItem);
	self.SelectIconLabel:SetMouseVisible(false);

	self.SelectIcon = Turbine.UI.Control();
	self.SelectIcon:SetParent(self.SelectIconLabel);
	self.SelectIcon:SetMouseVisible(false);
	self.SelectIcon:SetSize(Constants.SortIcon.Width, Constants.SortIcon.Height);
	self.SelectIcon:SetBackground(Constants.SortIcon.DropdownArrow);
	
	local getWindow = function(control)
		local window = control;
		local parent = window:GetParent();
		while parent ~= nil do
			window = parent;
			parent = window:GetParent();
		end
		
		return window;
	end
	
	self.SelectionList = Turbine.UI.Control();
	self.SelectionList:SetParent(getWindow(parent));
	self.SelectionList:SetZOrder(parent:GetZOrder() + 100);
	self.SelectionList:SetVisible(false);
	
	self.SelectionListBox = Turbine.UI.ListBox();
	self.SelectionListBox:SetParent(self.SelectionList);
	self.SelectionListBox:SetPosition(Constants.Border, Constants.Border);
	
	self.AutoSort = false;
	self.FireEvent = true;
	
	self.SelectionListBox.SelectedIndexChanged = function(sender, args)
		local selectedLabel = self.SelectionListBox:GetSelectedItem();
		local selectedText = selectedLabel:GetText();
		
		self.DisplayItemBox:SetText(selectedText);

		if self.FireEvent then
			self.SelectedIndexChanged(self, 
			{ 
				["SelectedIndex"] = self.SelectionListBox:GetSelectedIndex(),
				["SelectedValue"] = selectedLabel.Value,
				["SelectedText"] = selectedText;
			});
		end
		
		self.FireEvent = true;
	end
	
	self.SelectionListBox.FocusLost = function(sender, args)
		if not self.LinesScrollBar.HasFocus then
			self.SelectionList:SetVisible(false);
		end
	end
	
	self.SelectionList.VisibleChanged = function(sender, args)
		if self.SelectionList:IsVisible() then
			local control = self.DisplayItem;
			local left = control:GetLeft();
			local top = control:GetTop();
			local parent = control:GetParent();
			while parent:GetParent() ~= nil do
				control = parent;
				left = left + control:GetLeft()
				top = top + control:GetTop();
				
				parent = parent:GetParent();
			end
			self.SelectionList:SetPosition(left, top + self.DisplayItem:GetHeight() + Constants.Border);	
		end
	end
	
	self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.LinesScrollBar:SetParent(self.SelectionListBox);
	self.LinesScrollBar.HasFocus = false;
	self.SelectionListBox:SetVerticalScrollBar(self.LinesScrollBar);
	self.LinesScrollBar.MouseEnter = function(sender, args)
		self.LinesScrollBar.HasFocus = true;
	end
	self.LinesScrollBar.MouseLeave = function(sender, args)
		self.LinesScrollBar.HasFocus = false;
		self.SelectionListBox:Focus();
	end

	self.SelectedIndexChanged = function() end;
end

function DropdownList:SetAutoSort(sort)
	self.AutoSort = sort;
end

function DropdownList:SetPosition(left, top)
	self.DisplayItem:SetPosition(left, top);
end

function DropdownList:SetSize(width, height, dropdownHeight, heightInRows)
	self.HeightInRows = heightInRows;
	self.DropdownHeight = dropdownHeight;
	
	self.DisplayItem:SetSize(width, height);
	self.DisplayItemBox:SetSize(self.DisplayItem:GetWidth() - (Constants.Border * 2) - Constants.SortIcon.LabelWidth, self.DisplayItem:GetHeight() - (Constants.Border * 2));

	self.SelectIconLabel:SetSize(Constants.SortIcon.LabelWidth, self.DisplayItem:GetHeight() - (Constants.Border * 2));
	self.SelectIconLabel:SetPosition(self.DisplayItemBox:GetLeft() + self.DisplayItemBox:GetWidth(), Constants.Border);
	self.SelectIcon:SetPosition((self.SelectIconLabel:GetWidth() - Constants.SortIcon.Width) / 2, (self.SelectIconLabel:GetHeight() - Constants.SortIcon.Height) / 2);

	self.SelectionList:SetWidth(width);
	self.SelectionListBox:SetWidth(width - (Constants.Border * 2));

	for i = 1, self.SelectionListBox:GetItemCount() do
		item = self.SelectionListBox:GetItem(i):SetWidth(self.SelectionListBox:GetWidth());
	end
	
	self.LinesScrollBar:SetWidth(Constants.ScrollBarSize);
	self.LinesScrollBar:SetPosition(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize,  0);

	self:SetDropdownHeight();
end

function DropdownList:SetDropdownHeight()
	local dropdownHeight = self.DropdownHeight;
	if self.HeightInRows ~= nil then
		local itemCount = self.SelectionListBox:GetItemCount();
		if itemCount == 0 then
			itemCount = 1
		end
		if itemCount < self.HeightInRows then
			dropdownHeight = itemCount * self.DisplayItemBox:GetHeight();
		else
			dropdownHeight = self.HeightInRows * self.DisplayItemBox:GetHeight();
		end
		dropdownHeight = dropdownHeight + (Constants.Border * 2)
	end	
	
	self.SelectionList:SetHeight(dropdownHeight);
	self.SelectionListBox:SetHeight(dropdownHeight - (Constants.Border * 2));
	self.LinesScrollBar:SetHeight(self.SelectionListBox:GetHeight());
end

function DropdownList:SetWidth(width)
	self:SetSize(width, self.DisplayItem:GetHeight(), self.SelectionList:GetHeight());
end

function DropdownList:GetLeft()
	return self.DisplayItem:GetLeft();
end

function DropdownList:GetWidth()
	return self.DisplayItem:GetWidth();
end

function DropdownList:SetBorderColor(colour)
	self.DisplayItem:SetBackColor(colour);
	self.SelectionList:SetBackColor(colour);
end

function DropdownList:SetBackColor(colour)
	self.DisplayItemBox:SetBackColor(colour);
	self.SelectIconLabel:SetBackColor(colour);	
	self.SelectionListBox:SetBackColor(colour);	
end

function DropdownList:SetForeColor(colour)
	self.DisplayItemBox:SetForeColor(colour);
end

function DropdownList:SetFont(font)
	self.DisplayItemBox:SetFont(font);
end

function DropdownList:SetVisible(visible)
	self.DisplayItem:SetVisible(visible);
	self.SelectionListBox:SetVisible(visible);	
end

function DropdownList:GetSelectedIndex()
	return self.SelectionListBox:GetSelectedIndex();
end

function DropdownList:GetSelectedValue()
	local index = self.SelectionListBox:GetSelectedIndex();
	if index > 0 then
		return self.SelectionListBox:GetItem(index).Value;
	end
	
	return nil;
end

function DropdownList:GetSelectedText()
	local index = self.SelectionListBox:GetSelectedIndex();
	if index > 0 then
		return self.SelectionListBox:GetItem(index):GetText();
	end
	
	return nil;
end

function DropdownList:SetSelectedValue(value, fireEvent)
	local index = 0;
	local checkIndex = 1;
	local itemCount = self.SelectionListBox:GetItemCount();
	while checkIndex <= itemCount and index == 0 do
		if self.SelectionListBox:GetItem(checkIndex).Value == value then
			index = checkIndex;
		else
			checkIndex = checkIndex + 1;
		end
	end
	
	if index > 0 then
		self:SetSelectedIndex(index, fireEvent);
	end
end
	
function DropdownList:SetSelectedIndex(index, fireEvent)
	if index ~= self.SelectionListBox:GetSelectedIndex() then
		if fireEvent == nil then
			fireEvent = true;
		end
	
		self.FireEvent = fireEvent;
		self.SelectionListBox:SetSelectedIndex(index);
	end
end

function DropdownList:FindInsertIndex(text)
	local itemCount = self.SelectionListBox:GetItemCount();
	local insertIndex = itemCount + 1;
	for index = 1, itemCount do
		local checkText = self.SelectionListBox:GetItem(index):GetText();
		local compare = text:Compare(checkText);
		if compare == 0 then
			insertIndex = nil;
			break;
		elseif compare < 0 then
			insertIndex = index;
			break;
		end
	end

	return insertIndex;
end

function DropdownList:AddItem(text, value, selectNewItem)
	local insertIndex = self:GetItemCount() + 1;
	
	if self.AutoSort then
		insertIndex = self:FindInsertIndex(text);
		if insertIndex == nil then
			return;
		end
	end
	
	local item = Turbine.UI.Label();
	item:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	item:SetSize(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize, self.DisplayItemBox:GetHeight());
	item:SetFont(self.DisplayItemBox:GetFont());
	item:SetText(text);
	if value == nil then
		item.Value = text;
	else
		item.Value = value;
	end
	
	self.SelectionListBox:InsertItem(insertIndex, item);
	
	if selectNewItem then
		self.SelectionListBox:SetSelectedIndex(insertIndex);
	end
	
	item.MouseClick = function(sender, args)
		self.SelectionList:SetVisible(false);
	end
	
	item.MouseEnter = function(sender, args)
		item:SetOutlineColor(Constants.Colours.HoverOutline);
		item:SetFontStyle(Turbine.UI.FontStyle.Outline);
	end
	item.MouseLeave = function(sender, args)
		item:SetOutlineColor(Constants.Colours.NormalOutline);
		item:SetFontStyle(Turbine.UI.FontStyle.None);
	end
	
	self:SetDropdownHeight();
end

function DropdownList:UpdateSelectedItem(text, value)
	self.FireEvent = false;
	self:RemoveSelectedItem();
	self.FireEvent = false;
	self:AddItem(text, value, true);
end

function DropdownList:RemoveSelectedItem()
	local index = self.SelectionListBox:GetSelectedIndex();
	if index > 0 then
		self.SelectionListBox:RemoveItemAt(index);
		index = index - 1;
		if index > 0 then
			self.SelectionListBox:SetSelectedIndex(index);
		else
			if self.SelectionListBox:GetItemCount() == 0 then
				self.SelectionListBox:SetSelectedIndex(0);
			else
				self.SelectionListBox:SetSelectedIndex(1);
			end
		end
		
		self:SetDropdownHeight();
	end
end

function DropdownList:ClearItems()
	self.SelectionListBox:ClearItems();
	self.DisplayItemBox:SetText("");
	self:SetDropdownHeight();
end

function DropdownList:GetItemCount()
	return self.SelectionListBox:GetItemCount();
end