JoinWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 295,
		Height = 160,
		ExpandedHeight = 250
	},
	Padding = 
	{ 
		Left = 15, 
		Top = 55, 
		Right = 15, 
		Bottom = 20
	},
	Dropdown = 
	{
		Width = 160,
		Height = 20,
		OpenRows = 4
	},
	Spacing = 8,
	LineHeight = 20,
	LabelSize = 115,
	TextBoxSize = 145
}

function JoinWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self:SetText(Strings["join_channel"]);
	self:SetPosition(tonumber(Settings.JoinWindow.Position.Left), tonumber(Settings.JoinWindow.Position.Top));
	self:SetSize(Constants.MinimumSize.Width, Constants.MinimumSize.ExpandedHeight);
	self:SetOpacity(tonumber(Settings.JoinWindow.Opacity) / 100);
	
	local top = Constants.Padding.Top;
	local nextRow = Constants.LineHeight + Constants.Spacing;
	
	local joinChannelLabel = CreateLabel(self);
	joinChannelLabel:SetPosition(Constants.Padding.Left, top);
	joinChannelLabel:SetWidth(Constants.LabelSize);
	joinChannelLabel:SetText(Strings["channel_name"]);
	
	self.ChannelNameBox = CreateTextBox(self);
	self.ChannelNameBox:SetPosition(joinChannelLabel:GetLeft() + joinChannelLabel:GetWidth() + Constants.Spacing, top);
	self.ChannelNameBox:SetWidth(Constants.TextBoxSize);
	self.ChannelNameBox.KeyDown = function(sender, args) 
		if args.Action == 162 then
			self.PasswordBox:Focus();
		end
	end
	self.ChannelNameBox.FocusGained = function(sender, args)
		self.ChannelNameBox.SaveText = self.ChannelNameBox:GetText();
		self.ChannelNameBox:SetWantsUpdates(true);
	end
	self.ChannelNameBox.FocusLost = function(sender, args)
		self.ChannelNameBox:SetWantsUpdates(false);
		self.ChannelNameBox:SetText(self.ChannelNameBox:GetText():gsub(" ", ""));
	end
	self.ChannelNameBox.Update = function(sender, args)
		local newText = self.ChannelNameBox:GetText();
		if self.ChannelNameBox.SaveText ~= newText then
			self.ChannelNameBox.SaveText = newText;
			self:SetJoinButtonText();
		end
	end
	
	top = top + nextRow;
	
	local passwordLabel = CreateLabel(self);
	passwordLabel:SetPosition(Constants.Padding.Left, top);
	passwordLabel:SetWidth(Constants.LabelSize);
	passwordLabel:SetText(Strings["password"]);
	
	self.PasswordBox = CreateTextBox(self);
	self.PasswordBox:SetPosition(joinChannelLabel:GetLeft() + joinChannelLabel:GetWidth() + Constants.Spacing, top);
	self.PasswordBox:SetWidth(Constants.TextBoxSize);
	self.PasswordBox.FocusGained = function(sender, args)
		self.PasswordBox.SaveText = self.PasswordBox:GetText();
		self.PasswordBox:SetWantsUpdates(true);
	end
	self.PasswordBox.FocusLost = function(sender, args)
		self.PasswordBox:SetWantsUpdates(false);
		self.PasswordBox:SetText(self.PasswordBox:GetText():gsub(" ", ""));
	end
	self.PasswordBox.Update = function(sender, args)
		local newText = self.PasswordBox:GetText();
		if self.PasswordBox.SaveText ~= newText then
			self.PasswordBox.SaveText = newText;
			self:SetJoinButtonText();
		end
	end
	
	top = top + nextRow;
	
	self.PreviousLabel = CreateLabel(self);
	self.PreviousLabel:SetPosition(Constants.Padding.Left, top);
	self.PreviousLabel:SetSize(self:GetWidth() - Constants.Padding.Left - Constants.Padding.Right, self.PreviousLabel:GetHeight() * 2);
	self.PreviousLabel:SetMultiline(true);
	self.PreviousLabel:SetText(Strings["select_previous"]);
	
	top = top + self.PreviousLabel:GetHeight();
	
	self.PreviousChannel = CreateDropdown(self, left, top, Constants.Dropdown.Width, Constants.Dropdown.Height, nil, Constants.Dropdown.OpenRows);
	self.PreviousChannel:SetPosition(Constants.Padding.Left, top);
	self.PreviousChannel.SelectedIndexChanged = function(sender, args)
		local details = args.SelectedValue;
		self.ForgetButton:SetEnabled(self.PreviousChannel:GetSelectedIndex() > 1);
		self.ChannelNameBox:SetText(details.Name);
		self.PasswordBox:SetText(details.Password);
		self:SetJoinButtonText();
	end
	
	self.ForgetButton = CreateButton(self);
	self.ForgetButton:SetPosition(self.PreviousChannel:GetLeft() + self.PreviousChannel:GetWidth() + Constants.Spacing, top);
	self.ForgetButton:SetEnabled(false);
	self.ForgetButton:SetText(Strings["forget"]);
	self.ForgetButton.MouseClick = function(sender, args)
		local details = self.PreviousChannel:GetSelectedValue();
		Settings.PreviousChannels[details.Name:ToLower()] = nil;
		self.PreviousChannel:RemoveSelectedItem();
		self.PreviousChannel:SetSelectedIndex(1);
	end
	

	local center = self:GetWidth() / 2;
	
	self.CancelButton = ShortcutButton(true);
	self.CancelButton:SetParent(self);
	self.CancelButton:SetSize(Globals.Sizes.ShortcutButton.Width, Globals.Sizes.ShortcutButton.Height);
	self.CancelButton:SetLeft(center - Globals.Sizes.ShortcutButton.Width - Constants.Spacing / 2);
	self.CancelButton:SetIcons(Globals.Icons.ShortcutButton);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.QuickslotClicked = function(sender, args)
		self.JoinDetails = nil;
		self:Hide();
	end

	self.JoinButton = ShortcutButton();
	self.JoinButton:SetParent(self);
	self.JoinButton:SetSize(Globals.Sizes.ShortcutButton.Width, Globals.Sizes.ShortcutButton.Height);
	self.JoinButton:SetLeft(center + Constants.Spacing / 2);
	self.JoinButton:SetIcons(Globals.Icons.ShortcutButton);
	self.JoinButton:SetText(Strings["ok"]);
	self.JoinButton:SetEnabled(false);
	self.JoinButton.QuickslotClicked = function(sender, args)
		self.JoinDetails = 
		{ 
			ChannelName = self.ChannelNameBox:GetText():gsub(" ", ""),
			Password = self.PasswordBox:GetText():gsub(" ", ""),
			JoinTime = Turbine.Engine.GetLocalTime()
		}
		self:Hide();
	end
	
	self:LayoutControls();
end

function JoinWindow:LayoutControls()
	local top = self:GetHeight() - Constants.Padding.Bottom - Globals.Sizes.ShortcutButton.Height;
	self.CancelButton:SetTop(top);
	self.JoinButton:SetTop(top);
end

function JoinWindow:Show()
	self.ChannelNameBox:SetText("");
	self.PasswordBox:SetText("");
	self:SetJoinButtonText();
	
	self.PreviousChannel:ClearItems();
	self.PreviousChannel:AddItem(Strings["select"], {});
	
	local currentChatChannels = {};
	for index, channel in ipairs(userChatterWindow.ChatChannels) do
		currentChatChannels[channel.Name:ToLower()] = index;
	end
	
	for channelKey, channelInfo in pairs(Settings.PreviousChannels) do
		if currentChatChannels[channelKey] == nil then
			self.PreviousChannel:AddItem(channelInfo.Name, channelInfo);
		end
	end
	
	self.PreviousChannel:SetSelectedIndex(1);
	
	local expanded = (self.PreviousChannel:GetItemCount() > 1);
	
	self.PreviousLabel:SetVisible(expanded);
	self.PreviousChannel:SetVisible(expanded);
	self.ForgetButton:SetVisible(expanded);
	self.ForgetButton:SetEnabled(false);
	
	if expanded then
		self:SetHeight(Constants.MinimumSize.ExpandedHeight);
	else
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self:LayoutControls();
	
	self:SetVisible(true);
	self:Activate();
	self.ChannelNameBox:Focus();
end

function JoinWindow:Hide()
	self:SetVisible(false);
end

function JoinWindow:SetJoinButtonText()
	local channelName = self.ChannelNameBox:GetText():gsub(" ", "");
	if string.len(channelName) > 0 then
		local password = self.PasswordBox:GetText():gsub(" ", "");
		self.JoinButton:SetEnabled(true);
		self.JoinButton:SetShortcut("/" .. Strings["alias_join_channel"] .. " " .. (channelName .. " " .. password):Trim());
	else
		self.JoinButton:SetEnabled(false);
		self.JoinButton:SetShortcut("");
	end
end