import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay"

import "Simbo.UserChatter.Class";
import "Simbo.UserChatter.Events";
import "Simbo.UserChatter.Localisation";
import "Simbo.UserChatter.StringFunctions";
import "Simbo.UserChatter.WindowFunctions";
import "Simbo.UserChatter.WindowLotro";
import "Simbo.UserChatter.DropdownList";
import "Simbo.UserChatter.JoinWindow";
import "Simbo.UserChatter.UserChatterWindow";
import "Simbo.UserChatter";

Settings = 
{
	UserChatterWindow =
	{
		Position = {Left = "100", Top = "100"},
		Size = {Width = "400"},
		Opacity = "96"
	},
	JoinWindow =
	{
		Position = {Left = "100", Top = "100"},
		Opacity = "96"
	},
	PreviousChannels = {}
};

Globals =
{
	Sizes =
	{
		Label = { Height = 20 },
		TextBox = { Width = 200, Height = 20 },
		Button = { Width = 100, Height = 24 },
		WideButton = { Width = 120, Height = 24 },
		ExtraWideButton = { Width = 160, Height = 24 },
		NarrowButton = { Width = 55, Height = 24 },
		ShortcutButton = { Width = 100,	Height = 18 },
		ShortcutWideButton = { Width = 130,	Height = 18 }
	},
	Fonts = 
	{
		Label = Turbine.UI.Lotro.Font.Verdana14,
		SmallLabel = Turbine.UI.Lotro.Font.Verdana12, 
		TextBox = Turbine.UI.Lotro.Font.Verdana14,
		ListBox = Turbine.UI.Lotro.Font.Verdana14
	},
	Icons =
	{
		ShortcutButton = 
		{
			Normal = "Simbo/UserChatter/Icons/BlankButton.tga",
			Hover = "Simbo/UserChatter/Icons/BlankButtonHover.tga",
			Inactive = "Simbo/UserChatter/Icons/BlankButtonInactive.tga"
		},
		ShortcutWideButton = 
		{
			Normal = "Simbo/UserChatter/Icons/BlankWideButton.tga",
			Hover = "Simbo/UserChatter/Icons/BlankWideButtonHover.tga",
			Inactive = "Simbo/UserChatter/Icons/BlankWideButtonInactive.tga"
		}
	},
	Colours =
	{
		Label = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		InactiveLabel = Turbine.UI.Color(1.0, 0.4, 0.4, 0.4),
		TextBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		ListBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		Confirmation = Turbine.UI.Color((210 / 255), 1, 0),
		Warning = Turbine.UI.Color(1.0, 1.0, 0.0, 0.0)
	},
	BackgroundColours =
	{
		TextBox = Turbine.UI.Color(1.0, 0, 0, 0),
		ListBox = Turbine.UI.Color(1.0, 0, 0, 0)
	},
	BorderColours =
	{
		ListBox = Turbine.UI.Color(1.0, (41 / 255), (48 / 255), (72 / 255))
	}
}

UserChatterLoad = Turbine.PluginData.Load;
UserChatterSave = Turbine.PluginData.Save;

Settings = UserChatterLoad(Turbine.DataScope.Character, "UserChatterSettings") or Settings;
if Settings.PreviousChannels == nil then
	Settings.PreviousChannels = {};
end

userChatterWindow = Simbo.UserChatter.UserChatterWindow();
userChatterWindow:Show();

joinWindow = Simbo.UserChatter.JoinWindow();

UserChatterCommand = Turbine.ShellCommand();

function UserChatterCommand:Execute(cmd, args)
	if args ~= nil then
		args = args:lower();
	end

	if args == Strings["cmd_show"] then
		userChatterWindow:Show();
	elseif args == Strings["cmd_hide"] then
		userChatterWindow:Hide();
	elseif args == Strings["cmd_toggle"] then
		userChatterWindow:Toggle();
	else
		UserChatterCommand:GetHelp();
	end
end

function UserChatterCommand:GetHelp()
	Turbine.Shell.WriteLine(Strings["cmd_usage"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help1"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help2"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help3"]);
end

function UserChatter_ChatReceived(sender, args)
	local channelNumber = 0;
	local message = args.Message:gsub("\194\160", " ");
	if message:match(".+\n$") then
		message = message:sub(1, message:len() - 1);
	end
		
	if args.ChatType == Turbine.ChatType.Standard then
		if message == Strings["msg_current_channels"] or message == Strings["msg_not_in_channels"] then
			userChatterWindow:Initialise();
		elseif userChatterWindow:IsInitialised() then
			local joinName, joinNumber = string.match(message, "^" .. Strings["msg_joined_room"] .. "$");
			if joinName ~= nil then
				userChatterWindow:JoinChatChannel(tonumber(joinNumber), joinName, nil, false);
			end
		end
	end
	
	if not userChatterWindow:IsInitialised() then
		return
	end
	
	for number = 1, 4 do
		if args.ChatType == Turbine.ChatType["UserChat" .. number] then
			channelNumber = number;
			break;
		end
	end
	
	if channelNumber > 0 then
		local leaveChannel = string.match(message, "^" .. Strings["msg_left_room"] .. "$");
		if leaveChannel ~= nil then
			userChatterWindow:LeaveChatChannelNumber(channelNumber, leaveChannel, false);
		else
			local checkChannel, chatMessage = string.match(message, "^" .. Strings["msg_channel_details"] .. "$");
			if checkChannel == tostring(channelNumber) and chatMessage ~= nil then
				if chatMessage == Strings["msg_not_connected"] then
					userChatterWindow:LeaveChatChannelNumber(channelNumber, nil, true);
				else
					local channelName, password = string.match(chatMessage, "^" .. Strings["msg_details_with_password"] .. "$");
					if channelName == nil then
						channelName = string.match(chatMessage, "^" .. Strings["msg_details_without_password"] .. "$");
						password = "";
					end
					
					userChatterWindow:JoinChatChannel(channelNumber, channelName, password, true);
				end
			else
				local chatChannel, chatMessage = string.match(message, "^" .. Strings["msg_you_to_channel"] .."$");
				if chatChannel ~= nil then
					userChatterWindow:ProcessChat(channelNumber, chatChannel);
				else
					local chatName;
					chatChannel, chatName, chatMessage = string.match(message, "^" .. Strings["msg_other_to_channel"] .. "$");
					if chatChannel ~= nil then
						--local playerName = string.match(chatName, "^<Select:IID:0x[0-9A-F]+>(%S+)<\\Select>$");
						userChatterWindow:ProcessChat(channelNumber, chatChannel);
					end
				end
			end
		end
	end
end

function ShowUserChatterMessage(message, level)
	local display;
	if level == nil or level == 0 then
		display = "";
	elseif level == 1 then
		display = "<rgb=#00FF00>";
	elseif level == 2 then
		display = "<rgb=#FF0000>";
	end
	
	display = display .. message;
	
	if level ~= nil and level > 0 then
		display = display .. "</rgb>";
	end
	
	Turbine.Shell.WriteLine(display);
end

AddCallback(Turbine.Chat, "Received", UserChatter_ChatReceived);

Plugins["User Chatter"].Unload = function(sender, args)
	RemoveCallback(Turbine.Chat, "Received", UserChatter_ChatReceived);
	userChatterWindow:SaveSettings();
end

Turbine.Shell.AddCommand("userchatter", UserChatterCommand);
Turbine.Shell.WriteLine(Strings["userchatter"] .. " v"..Plugins["User Chatter"]:GetVersion().." by Simbo");