ShortcutButton = class(Turbine.UI.Control);

local Constants =
{
	SlotAdjust = -2,
	Font = Turbine.UI.Lotro.Font.TrajanPro14,
	Colours = 
	{
		Active = Turbine.UI.Color((249 / 255), (227 / 255), (44 / 255)),
		Inactive = Turbine.UI.Color(1.0, 0.8, 0.8, 0.8),
		Warn = Turbine.UI.Color(1.0, 1.0, 0.0, 0.0) 
	}
}

function ShortcutButton:Constructor(buttonOnly)
	Turbine.UI.Control.Constructor(self);
	
	self.Icons = {};
	self.ButtonOnly = buttonOnly or false;
	self.WarnMode = false;
	
	self.Shortcut = Turbine.UI.Lotro.Shortcut();
	self.Shortcut:SetType(Turbine.UI.Lotro.ShortcutType.Alias);
	self.Shortcut:SetData("");
	self.ShortcutData = "";
	
	self.Quickslot = Turbine.UI.Lotro.Quickslot();
	self.Quickslot:SetParent(self);
	self.Quickslot:SetPosition(0, Constants.SlotAdjust);
	self.Quickslot:SetAllowDrop(false);
	self.Quickslot:SetShortcut(self.Shortcut);
	self.Quickslot:SetVisible(false);
	self.Quickslot.MouseClick = function(sender, args)
		self.QuickslotClicked(self, args);
	end
	
	-- Fix for dragging short-cut
	self.Quickslot.DragDrop = function(sender, args)
		self.Quickslot:SetShortcut(self.Shortcut); -- restore shortcut
	end
	
	self.QuickslotIcon = Turbine.UI.Control();
	self.QuickslotIcon:SetParent(self);
	self.QuickslotIcon:SetPosition(0, 0);
	self.QuickslotIcon:SetZOrder(self.Quickslot:GetZOrder() + 1);
	self.QuickslotIcon:SetMouseVisible(self.ButtonOnly);
	self.QuickslotIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.QuickslotIcon.EnabledChanged = function(sender, args)
		self:DisplayIcon();
	end
	self.QuickslotIcon.MouseEnter = function(sender, args)
		if self.ButtonOnly and self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(self.Icons.Hover);
		end
	end
	self.QuickslotIcon.MouseLeave = function(sender, args)
		if self.ButtonOnly and self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(self.Icons.Normal);
		end
	end
	self.QuickslotIcon.MouseDown = function(sender, args)
		if self.ButtonOnly and self.QuickslotIcon:IsEnabled() then
			self.QuickslotLabel:SetForeColor(Constants.Colours.Inactive);
		end
	end
	self.QuickslotIcon.MouseUp = function(sender, args)
		if self.ButtonOnly and self.QuickslotIcon:IsEnabled() then
			self:ActiveLabelColour();
		end
	end
	self.QuickslotIcon.MouseClick = function(sender, args)
		if self.ButtonOnly and self.QuickslotIcon:IsEnabled() then
			self.QuickslotClicked(self, args);
		end
	end
	
	self.QuickslotLabel = CreateLabel(self, false, 0, 0);
	self.QuickslotLabel:SetZOrder(self.Quickslot:GetZOrder() + 2);
	self.QuickslotLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.QuickslotLabel:SetFont(Constants.Font);
	self.QuickslotLabel:SetForeColor(Constants.Colours.Active);
	self.QuickslotLabel:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.QuickslotLabel:SetText("");
	self.Quickslot.MouseEnter = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(self.Icons.Hover);
		end
	end
	self.Quickslot.MouseLeave = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(self.Icons.Normal);
		end
	end
	self.Quickslot.MouseDown = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotLabel:SetForeColor(Constants.Colours.Inactive);
		end
	end
	self.Quickslot.MouseUp = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self:ActiveLabelColour();
		end
	end
	
	self.SizeChanged = function(sender, args)
		self.QuickslotIcon:SetSize(self:GetWidth(), self:GetHeight());
		self.Quickslot:SetSize(self:GetWidth() - 1, self:GetHeight());
		self.QuickslotLabel:SetSize(self:GetWidth(), self:GetHeight());
	end
	
	self.QuickslotClicked = function() end;
end

function ShortcutButton:SetIcons(set)
	self.Icons.Normal = set.Normal;
	self.Icons.Hover = set.Hover;
	self.Icons.Inactive = set.Inactive;
	self:DisplayIcon();
end

function ShortcutButton:SetWarnMode(warnMode)
	self.WarnMode = warnMode;
	if self.QuickslotIcon:IsEnabled() then
		self:ActiveLabelColour();
	end
end
		
function ShortcutButton:ActiveLabelColour()
	if self.WarnMode then
		self.QuickslotLabel:SetForeColor(Constants.Colours.Warn);
	else
		self.QuickslotLabel:SetForeColor(Constants.Colours.Active);
	end
end
		
function ShortcutButton:DisplayIcon()
	if self.QuickslotIcon:IsEnabled() then
		self.QuickslotIcon:SetBackground(self.Icons.Normal);
		self:ActiveLabelColour();
	else
		self.QuickslotIcon:SetBackground(self.Icons.Inactive);
		self.QuickslotLabel:SetForeColor(Constants.Colours.Inactive);
	end
end

function ShortcutButton:SetShortcut(shortcutAlias)
	if shortcutAlias ~= nil then
		if shortcutAlias ~= "" then
			self.Shortcut:SetData(shortcutAlias);
		end
		self.ShortcutData = shortcutAlias;
		self.Quickslot:SetVisible(self.QuickslotIcon:IsEnabled() and shortcutAlias ~= "" and not self.ButtonOnly);
	end
	self.Quickslot:SetShortcut(self.Shortcut);
end

function ShortcutButton:SetEnabled(enabled)
	self.Quickslot:SetVisible(enabled and self.ShortcutData ~= "" and not self.ButtonOnly);
	self.QuickslotIcon:SetEnabled(enabled);
end

function ShortcutButton:SetVisible(visible)
	self.Quickslot:SetVisible(visible and self.ShortcutData ~= "" and not self.ButtonOnly);
	self.QuickslotIcon:SetVisible(visible);
	self.QuickslotLabel:SetVisible(visible);
end

function ShortcutButton:SetText(text)
	self.QuickslotLabel:SetText(text);
end