function string:ToLower()
	if self == nil or string.len(self) == 0 then
		return self;
	end
	
	local byteIndex = 1;
	local lowerCase = {};
	
	while byteIndex <= #self do
		local byteValue = string.byte(self, byteIndex);
		if byteValue >= 192 then
			local checkString =  string.sub(self, byteIndex, byteIndex + 1);
			local map = LowerCaseMap[checkString];
			if map ~= nil then
				table.insert(lowerCase, map);
			else
				table.insert(lowerCase, checkString);
			end
			byteIndex = byteIndex + 1;
		else
			table.insert(lowerCase, string.lower(string.char(byteValue))); 
		end
		
		byteIndex = byteIndex + 1;
	end
	
	return table.concat(lowerCase);
end

function string:Trim()
	local from = self:match"^%s*()";
	return from > #self and "" or self:match(".*%S", from);
end

function string:Equals(text)
	return string.ToLower(self) == string.ToLower(text);
end

function string:Compare(text)
	local lowerSelf = string.ToLower(self);
	local lowerText = string.ToLower(text);
	if lowerSelf < lowerText then
		return -1;
	elseif lowerSelf == lowerText then
		return 0;
	else
		return 1;
	end
end

function string:Split(separator)
	local fields = {}
	
	separator = separator or ":";
	local pattern = string.format("([^%s]+)", separator);
	
	self:gsub(pattern, function(c) 
		if c:Trim() ~= "" then
			fields[#fields+1] = c:Trim() 
		end
	end);
	
	return fields;
end

function string:PadLeft(size, padding)
	local length = string.len(self);
	
	if length >= size then
		return self;
	end
	
	return padding:rep(size - length) .. self;
end

function string:StartsWith(text)
	if self == nil or text == nil or string.len(self) == 0 then
		return false;
	end
	
	local textLength = string.len(text);
	
	if textLength == 0 or textLength > string.len(self) then
		return false;
	end
	
	return string.ToLower(self:sub(1, textLength)) == string.ToLower(text);
end

function string:EndsWith(text)
	if self == nil or text == nil or string.len(self) == 0 then
		return false;
	end
	
	local textLength = string.len(text);
	local stringLength = string.len(self);
	
	if textLength == 0 or textLength > stringLength then
		return false;
	end
	
	return string.ToLower(self:sub(stringLength - textLength + 1)) == string.ToLower(text);
end

LowerCaseMap = {
	["\195\128"] = "\195\160", 
	["\195\129"] = "\195\161", 
	["\195\130"] = "\195\162", 
	["\195\131"] = "\195\163", 
	["\195\132"] = "\195\164", 
	["\195\133"] = "\195\165", 
	["\195\134"] = "\195\166", 
	["\195\135"] = "\195\167", 
	["\195\136"] = "\195\168", 
	["\195\137"] = "\195\169", 
	["\195\138"] = "\195\170", 
	["\195\139"] = "\195\171", 
	["\195\140"] = "\195\172", 
	["\195\141"] = "\195\173", 
	["\195\142"] = "\195\174", 
	["\195\143"] = "\195\175", 
	["\195\144"] = "\195\176", 
	["\195\145"] = "\195\177", 
	["\195\146"] = "\195\178", 
	["\195\147"] = "\195\179", 
	["\195\148"] = "\195\180", 
	["\195\149"] = "\195\181", 
	["\195\150"] = "\195\182", 
	["\195\152"] = "\195\184", 
	["\195\153"] = "\195\185", 
	["\195\154"] = "\195\186", 
	["\195\155"] = "\195\187", 
	["\195\156"] = "\195\188", 
	["\195\157"] = "\195\189", 
	["\195\158"] = "\195\190", 
	["\197\184"] = "\195\191", 
	["\196\128"] = "\196\129", 
	["\196\130"] = "\196\131", 
	["\196\132"] = "\196\133", 
	["\196\134"] = "\196\135", 
	["\196\136"] = "\196\137", 
	["\196\138"] = "\196\139", 
	["\196\140"] = "\196\141", 
	["\196\142"] = "\196\143", 
	["\196\144"] = "\196\145", 
	["\196\146"] = "\196\147", 
	["\196\148"] = "\196\149", 
	["\196\150"] = "\196\151", 
	["\196\152"] = "\196\153", 
	["\196\154"] = "\196\155", 
	["\196\156"] = "\196\157", 
	["\196\158"] = "\196\159", 
	["\196\160"] = "\196\161", 
	["\196\162"] = "\196\163", 
	["\196\164"] = "\196\165", 
	["\196\166"] = "\196\167", 
	["\196\168"] = "\196\169", 
	["\196\170"] = "\196\171", 
	["\196\172"] = "\196\173", 
	["\196\174"] = "\196\175", 
	["\196\176"] = "\196\177", 
	["\196\178"] = "\196\179", 
	["\196\180"] = "\196\181", 
	["\196\182"] = "\196\183", 
	["\196\185"] = "\196\186", 
	["\196\187"] = "\196\188", 
	["\196\189"] = "\196\190", 
	["\196\191"] = "\197\128", 
	["\197\129"] = "\197\130", 
	["\197\131"] = "\197\132", 
	["\197\133"] = "\197\134", 
	["\197\135"] = "\197\136", 
	["\197\138"] = "\197\139", 
	["\197\140"] = "\197\141", 
	["\197\142"] = "\197\143", 
	["\197\144"] = "\197\145", 
	["\197\146"] = "\197\147", 
	["\197\148"] = "\197\149", 
	["\197\150"] = "\197\151", 
	["\197\152"] = "\197\153", 
	["\197\154"] = "\197\155", 
	["\197\156"] = "\197\157", 
	["\197\158"] = "\197\159", 
	["\197\160"] = "\197\161", 
	["\197\162"] = "\197\163", 
	["\197\164"] = "\197\165", 
	["\197\166"] = "\197\167", 
	["\197\168"] = "\197\169", 
	["\197\170"] = "\197\171", 
	["\197\172"] = "\197\173", 
	["\197\174"] = "\197\175", 
	["\197\176"] = "\197\177", 
	["\197\178"] = "\197\179", 
	["\197\180"] = "\197\181", 
	["\197\182"] = "\197\183", 
	["\197\185"] = "\197\186", 
	["\197\187"] = "\197\188", 
	["\197\189"] = "\197\190"
}
