TabButton = class(Turbine.UI.Label);

local Constants = 
{
	Size = 
	{
		Width = 125,
		Height = 22
	},
	Colours =
	{
		Normal = Turbine.UI.Color((210 / 255), 1, 0),
		Selected = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		Inactive = Turbine.UI.Color(1.0, 0.7, 0.7, 0.7)
	},
	Backgrounds =
	{
		Normal = "Simbo/UserChatter/Icons/Tab.tga",
		Highlight = "Simbo/UserChatter/Icons/TabHighlight.tga"
	},
	Font = 
	{
		Normal = Turbine.UI.Lotro.Font.TrajanPro14,
		Selected = Turbine.UI.Lotro.Font.TrajanPro15
	}
}

function TabButton:Constructor()
	Turbine.UI.Label.Constructor(self);
	
	self.IsSelected = false;
	self.IsActive = false;
	self.IsHover = false;
	
	self:SetSize(Constants.Size.Width, Constants.Size.Height);
	self:SetPosition(0, 0);
	self:SetBackground(Constants.Backgrounds.Normal);
	self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self:SetFont(Constants.Font.Normal);
	self:SetForeColor(Constants.Colours.Inactive);
	
	self.MouseEnter = function(sender, args)
		self.IsHover = true;
		if not self.IsSelected then
			self:SetBackground(Constants.Backgrounds.Highlight);
		end
	end	
	self.MouseLeave = function(sender, args)
		self.IsHover = false;
		if not self.IsSelected then
			self:SetBackground(Constants.Backgrounds.Normal);
		end
	end
	self.MouseClick = function(sender, args)
		if not self.IsSelected then
			self:SetSelected(true);
			self.Selected(self, args);
		end
	end
	
	self.Selected = function() end;
end

function TabButton:SetSelected(selected)
	self.IsSelected = selected;
	if selected then
		self:SetBackground(Constants.Backgrounds.Highlight);
		self:SetFont(Constants.Font.Selected);
	else
		if not self.IsHover then
			self:SetBackground(Constants.Backgrounds.Normal);
		end
		self:SetFont(Constants.Font.Normal);
	end
	self:SetTabColor();
end

function TabButton:SetActive(active)
	self.IsActive = active;
	self:SetTabColor();
end

function TabButton:SetTabColor()
	if self.IsActive then
		if selected then
			self:SetForeColor(Constants.Colours.Selected);
		else
			self:SetForeColor(Constants.Colours.Normal);
		end
	else
		self:SetForeColor(Constants.Colours.Inactive);
	end
	self:SetText(self:GetText());
end