import "Simbo.UserChatter.ChatChannel";
import "Simbo.UserChatter.ShortcutButton";
import "Simbo.UserChatter.TabButton";

UserChatterWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 536,
		Height = 146
	},
	Padding =
	{
		Top = 40,
		Left = 15,
		Right = 15,
		Bottom = 30,
	},
	Sizes =
	{
		ChatLabel = 60,
		WideChatLabel = 80,
		ChatButton = 
		{
			Width = 30,
			Height = 20
		}
	},
	Icons =
	{
		Chat =
		{
			Normal = "Simbo/UserChatter/Icons/Chat.tga",
			Hover = "Simbo/UserChatter/Icons/ChatHover.tga",
			Inactive = "Simbo/UserChatter/Icons/ChatInactive.tga"
		}
	},
	TabSpacing = 2,
	LineSpacing = 8,
	MaxChannels = 4
}

function UserChatterWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.Initialised = false;
	self.AllowVerticalDrag = false;
	
	self.ChatChannels = {};
	self.CurrentChannelNumber = 1;
	for channelNumber = 1, Constants.MaxChannels do
		local button = TabButton();
		button:SetParent(self);
		self.ChatChannels[channelNumber] = ChatChannel(channelNumber, button);
	end
	
	self:SetText(Strings["userchatter"]);
	self:SetPosition(tonumber(Settings.UserChatterWindow.Position.Left), tonumber(Settings.UserChatterWindow.Position.Top));

	self:SetSize(tonumber(Settings.UserChatterWindow.Size.Width), Constants.MinimumSize.Height);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	
	self:SetOpacity(tonumber(Settings.UserChatterWindow.Opacity) / 100);
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
	
	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
	self.InitLabel = CreateLabel(self);
	self.InitLabel:SetPosition(left, top);
	self.InitLabel:SetWidth(Constants.MinimumSize.Width);
	self.InitLabel:SetText(Strings["click_refresh"]);
	
	local shortcutSizes = UIOptions["wide_channel_buttons"] and Globals.Sizes.ShortcutWideButton or Globals.Sizes.ShortcutButton;
	local shortcutImages = UIOptions["wide_channel_buttons"] and Globals.Icons.ShortcutWideButton or Globals.Icons.ShortcutButton;
	
	self.JoinChannelButton = ShortcutButton(true);
	self.JoinChannelButton:SetParent(self);
	self.JoinChannelButton:SetPosition(left, top);
	self.JoinChannelButton:SetSize(shortcutSizes.Width, shortcutSizes.Height);
	self.JoinChannelButton:SetIcons(shortcutImages);
	self.JoinChannelButton:SetText(Strings["join_channel"]);
	self.JoinChannelButton:SetVisible(false);
	self.JoinChannelButton.QuickslotClicked = function(sender, args)
		joinWindow:Show();
	end

	left = left + shortcutSizes.Width + Constants.TabSpacing;
	
	self.LeaveChannelButton = ShortcutButton();
	self.LeaveChannelButton:SetParent(self);
	self.LeaveChannelButton:SetPosition(left, top);
	self.LeaveChannelButton:SetSize(shortcutSizes.Width, shortcutSizes.Height);
	self.LeaveChannelButton:SetIcons(shortcutImages);
	self.LeaveChannelButton:SetShortcut("");
	self.LeaveChannelButton:SetText(Strings["leave_channel"]);
	self.LeaveChannelButton:SetVisible(false);
	
	left = left + shortcutSizes.Width + Constants.TabSpacing;
	
	self.ChannelInfoButton = ShortcutButton(true);
	self.ChannelInfoButton:SetParent(self);
	self.ChannelInfoButton:SetPosition(left, top);
	self.ChannelInfoButton:SetSize(shortcutSizes.Width, shortcutSizes.Height);
	self.ChannelInfoButton:SetIcons(shortcutImages);
	self.ChannelInfoButton:SetText(Strings["channel_info"]);
	self.ChannelInfoButton:SetVisible(false);
	self.ChannelInfoButton.QuickslotClicked = function(sender, args)
		local channel = self.ChatChannels[self.CurrentChannelNumber];
		local message = "";
		if channel.Password == nil then
			message = Strings["channel_details_with_unknownpassword"];
		elseif channel.Password == "" then
			message = Strings["channel_details"];
		else
			message = Strings["channel_details_with_password"];
		end
		Turbine.Shell.WriteLine(string.format(message, channel.Number, channel.Name, channel.Password or ""));
	end
	
	self.ListQuickslot = ShortcutButton();
	self.ListQuickslot:SetParent(self);
	self.ListQuickslot:SetTop(top);
	self.ListQuickslot:SetSize(Globals.Sizes.ShortcutButton.Width, Globals.Sizes.ShortcutButton.Height);
	self.ListQuickslot:SetIcons(Globals.Icons.ShortcutButton);
	self.ListQuickslot:SetText(Strings["refresh"]);
	self.ListQuickslot:SetShortcut("/" .. Strings["alias_list_channels"]);
	self.ListQuickslot.QuickslotClicked = function(sender, args)
		self:SetSyncStatus(true);
	end
	self:SetSyncStatus(true);
	
	top = top + self.JoinChannelButton:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	for channelNumber = 1, Constants.MaxChannels do
		local button = self.ChatChannels[channelNumber].Button;
		button:SetPosition(left, top);
		button:SetZOrder(2);
		button:SetSelected(channelNumber == 1);
		button.ChannelNumber = channelNumber;
		SetEmptyChannelText(button, channelNumber);
		button.Selected = function(sender, args) 
			self.CurrentChannelNumber = sender.ChannelNumber;
			
			local channel = self.ChatChannels[self.CurrentChannelNumber];
			
			self:SetChatBoxText(channel.ChatText, true, false);
			self:SetChannelFunctions(channel);
			
			for index = 1, Constants.MaxChannels do
				if index ~= sender.ChannelNumber then
					self.ChatChannels[index].Button:SetSelected(false);
				end
			end
		end
		left = left + button:GetWidth() + Constants.TabSpacing;
	end

	left = Constants.Padding.Left;
	top = top + self.ChatChannels[1].Button:GetHeight() + Constants.LineSpacing;

	local labelWidth = UIOptions["wide_chat_label"] and Constants.Sizes.WideChatLabel or Constants.Sizes.ChatLabel;
	
	self.ChatLabel = CreateLabel(self);
	self.ChatLabel:SetPosition(left, top);
	self.ChatLabel:SetWidth(labelWidth);
	self.ChatLabel:SetText(Strings["chat"]);
	self.ChatLabel:SetVisible(false);
	
	left = left + self.ChatLabel:GetWidth();
	
	self.ChatTextBox = CreateTextBox(self);
	self.ChatTextBox:SetPosition(left, top);
	self.ChatTextBox:SetVisible(false);
	
	left = left + self.ChatTextBox:GetWidth();
	
	self.ChatQuickslot = ShortcutButton();
	self.ChatQuickslot:SetParent(self);
	self.ChatQuickslot:SetTop(top);
	self.ChatQuickslot:SetSize(Constants.Sizes.ChatButton.Width, Constants.Sizes.ChatButton.Height);
	self.ChatQuickslot:SetEnabled(false);
	self.ChatQuickslot:SetIcons(Constants.Icons.Chat);
	self.ChatQuickslot:SetVisible(false);
	self.ChatQuickslot.QuickslotClicked = function(sender, args)
		self:SetChatBoxText("", true, true);
	end
	
	self.ChatTextBox.FocusGained = function(sender, args)
		self.ChatTextBox.SaveText = self.ChatTextBox:GetText();
		self.ChatTextBox:SetWantsUpdates(true);
	end
	self.ChatTextBox.FocusLost = function(sender, args)
		self.ChatTextBox:SetWantsUpdates(false);
	end
	self.ChatTextBox.Update = function(sender, args)
		local newText = self.ChatTextBox:GetText();
		if self.ChatTextBox.SaveText ~= newText then
			self.ChatTextBox.SaveText = newText;
			self:SetChatBoxText(newText, false, true);
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if self:IsVisible() then
			CheckWindowPosition(self);
		end
	end

	self:LayoutControls();
end

function UserChatterWindow:LayoutControls()
	self.ListQuickslot:SetLeft(self:GetWidth() - Constants.Padding.Right - self.ListQuickslot:GetWidth());

	self.ChatQuickslot:SetLeft(self:GetWidth() - Constants.Padding.Right - self.ChatQuickslot:GetWidth());
	self.ChatTextBox:SetWidth(self.ChatQuickslot:GetLeft() - self.ChatTextBox:GetLeft());
end

function UserChatterWindow:Show()
	self:SetVisible(true);
	self:Activate();
end

function UserChatterWindow:Hide()
	if self:IsVisible() then
		self:SetVisible(false);
	end
end

function UserChatterWindow:Toggle()
	if self:IsVisible() then
		self:Hide();
	else
		self:Show();
	end
end

function UserChatterWindow:SaveSettings()
	Settings.UserChatterWindow.Position.Top = tostring(self:GetTop());
	Settings.UserChatterWindow.Position.Left = tostring(self:GetLeft());
	Settings.UserChatterWindow.Size.Width = tostring(self:GetWidth());

	Settings.JoinWindow.Position.Top = tostring(joinWindow:GetTop());
	Settings.JoinWindow.Position.Left = tostring(joinWindow:GetLeft());

	UserChatterSave(Turbine.DataScope.Character, "UserChatterSettings", Settings);
end

function UserChatterWindow:IsInitialised()
	return self.Initialised;
end

function UserChatterWindow:SetChatBoxText(chatText, updateTextBox, updateChannel)
	if updateTextBox then
		self.ChatTextBox:SetText(chatText);
	end
	if updateChannel then
		self.ChatChannels[self.CurrentChannelNumber].ChatText = chatText;
	end
	if string.len(chatText) > 0 then
		self.ChatQuickslot:SetShortcut("/" .. self.CurrentChannelNumber .. " " .. chatText);
	else
		self.ChatQuickslot:SetShortcut("");
	end
end

function UserChatterWindow:Initialise()
	self.Initialised = true;
	self.InSync = true;
	
	self.InitLabel:SetVisible(false);
	
	self.JoinChannelButton:SetVisible(true);
	self.LeaveChannelButton:SetVisible(true);
	self.ChannelInfoButton:SetVisible(true);
	self.ChatLabel:SetVisible(true);
	self.ChatTextBox:SetVisible(true);
	self.ChatQuickslot:SetVisible(true);
	
	self:SetChannelFunctions(self.ChatChannels[self.CurrentChannelNumber]);
end

function UserChatterWindow:SetChannelFunctions(channel)
	local isConnected = channel:IsConnected();
	if channel:IsConnected() then
		self.LeaveChannelButton:SetShortcut("/" .. Strings["alias_leave_channel"] .. " " .. channel.Name);
	end
	
	self.LeaveChannelButton:SetEnabled(isConnected);
	self.ChannelInfoButton:SetEnabled(isConnected);
	self.ChatQuickslot:SetEnabled(channel:IsConnected());
	self.ChatTextBox:SetEnabled(channel:IsConnected());
	if channel:IsConnected() then
		self.ChatLabel:SetForeColor(Globals.Colours.Label);
		self.ChatLabel:SetText(Strings["chat"].. ": /" .. tostring(channel.Number));
	else
		self.ChatLabel:SetForeColor(Globals.Colours.InactiveLabel);
		self.ChatLabel:SetText(Strings["chat"]);
	end
end

function UserChatterWindow:ProcessChat(channelNumber, expectedChannelName)
	local channel = self.ChatChannels[channelNumber];
	if not channel:IsConnected() or string.Compare(channel.Name, expectedChannelName) ~= 0 then
		self:SetSyncStatus(false);
	end
end

function UserChatterWindow:JoinChatChannel(channelNumber, channelName, password, isRefresh)
	local channel = self.ChatChannels[channelNumber];
	
	if not isRefresh then
		if channel:IsConnected() then
			for checkChannel = 1, Constants.MaxChannels do
				if not self.ChatChannels[checkChannel]:IsConnected() then
					channelNumber = checkChannel;
					channel = self.ChatChannels[channelNumber];
					break;
				end
			end
			self:SetSyncStatus(false);
		end
	end
	
	channel.Name = channelName;
	if password == nil then
		local joinDetails = joinWindow.JoinDetails;
		if joinDetails ~= nil and string.Compare(joinDetails.ChannelName, channelName) == 0 then
			local checkTime = Turbine.Engine.GetLocalTime();
			if checkTime <= joinDetails.JoinTime + 10 then
				password = joinDetails.Password;
				joinWindow.JoinDetails = nil;
			end
		end
	end
	channel.Password = password;
	channel.Connected = true;
	
	if password ~= nil then
		Settings.PreviousChannels[channelName:ToLower()] = { Name = channelName, Password = password };
	end
	
	local button = channel.Button;
	button:SetActive(true);
	SetChannelText(button, channelName);
	
	if channelNumber == self.CurrentChannelNumber then
		self:SetChannelFunctions(channel);
	end
	
	self:CheckJoinButton();
end

function UserChatterWindow:LeaveChatChannelNumber(channelNumber, expectedChannelName, isRefresh)
	local channel = self.ChatChannels[channelNumber];
	local button = channel.Button;
	
	if not isRefresh then
		if not channel:IsConnected() or string.Compare(channel.Name, expectedChannelName) ~= 0 then
			self:SetSyncStatus(false);
		end
	end
	
	channel:Disconnect();

	button:SetActive(false);
	SetEmptyChannelText(button, channelNumber);

	if channelNumber == self.CurrentChannelNumber then
		self:SetChatBoxText("", true, true);
		self:SetChannelFunctions(channel);
	end
	
	self:CheckJoinButton();
end

function UserChatterWindow:CheckJoinButton()
	local allConnected = true;
	for index = 1, Constants.MaxChannels do
		if not self.ChatChannels[index]:IsConnected() then
			allConnected = false;
			break;
		end
	end
	
	self.JoinChannelButton:SetEnabled(not allConnected);
end

function UserChatterWindow:SetSyncStatus(inSync)
	self.InSync = inSync;
	if not inSync then
		ShowUserChatterMessage(Strings["out_of_sync"], 2);
		self.ListQuickslot:SetWarnMode(true);
	else
		self.ListQuickslot:SetWarnMode(false);
	end
end

function SetEmptyChannelText(button, channelNumber)
	SetChannelText(button, Strings["not_connected"])
end

function SetChannelText(button, channelName)
	button:SetText(channelName);
end


